'use strict';

var module$1 = require('module');
var eff = require('@eslint-react/eff');
var tsPattern = require('ts-pattern');
var kit = require('@eslint-react/kit');
var v4 = require('zod/v4');

var _documentCurrentScript = typeof document !== 'undefined' ? document.currentScript : null;
function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var module__default = /*#__PURE__*/_interopDefault(module$1);

// src/constants.ts
var NPM_SCOPE = "@eslint-react";
var GITHUB_URL = "https://github.com/Rel1cx/eslint-react";
var WEBSITE_URL = "https://eslint-react.xyz";

// src/get-doc-url.ts
var getDocsUrl = (pluginName) => (ruleName) => {
  if (pluginName === "x") {
    return `${WEBSITE_URL}/docs/rules/${ruleName}`;
  }
  return `${WEBSITE_URL}/docs/rules/${pluginName}-${ruleName}`;
};

// src/get-id.ts
var id = 0n;
var getId = () => (id++).toString();
var _require = module__default.default.createRequire((typeof document === 'undefined' ? require('u' + 'rl').pathToFileURL(__filename).href : (_documentCurrentScript && _documentCurrentScript.tagName.toUpperCase() === 'SCRIPT' && _documentCurrentScript.src || new URL('index.js', document.baseURI).href)));
function getReactVersion(fallback) {
  try {
    return tsPattern.match(_require("react")).with({ version: tsPattern.P.select(tsPattern.P.string) }, eff.identity).otherwise(() => fallback);
  } catch {
    return fallback;
  }
}
var CustomComponentPropSchema = v4.z.object({
  /**
   * The name of the prop in the user-defined component.
   * @example
   * "to"
   */
  name: v4.z.string(),
  /**
   * The name of the prop in the host component.
   * @example
   * "href"
   */
  as: v4.z.optional(v4.z.string()),
  /**
   * Whether the prop is controlled or not in the user-defined component.
   * @internal
   * @example
   * `true`
   */
  controlled: v4.z.optional(v4.z.boolean()),
  /**
   * The default value of the prop in the user-defined component.
   * @example
   * `"/"`
   */
  defaultValue: v4.z.optional(v4.z.string())
});
var CustomComponentSchema = v4.z.object({
  /**
   * The name of the user-defined component.
   * @example
   * "Link"
   */
  name: v4.z.string(),
  /**
   * The name of the host component that the user-defined component represents.
   * @example
   * "a"
   */
  as: v4.z.optional(v4.z.string()),
  /**
   * Attributes mapping between the user-defined component and the host component.
   * @example
   * `Link` component has a `to` attribute that represents the `href` attribute in the built-in `a` element with a default value of `"/"`.
   */
  attributes: v4.z.optional(v4.z.array(CustomComponentPropSchema)),
  /**
   * The ESQuery selector to select the component precisely.
   * @internal
   * @example
   * `JSXElement:has(JSXAttribute[name.name='component'][value.value='a'])`
   */
  selector: v4.z.optional(v4.z.string())
});
var CustomHooksSchema = v4.z.object({
  use: v4.z.optional(v4.z.array(v4.z.string())),
  useActionState: v4.z.optional(v4.z.array(v4.z.string())),
  useCallback: v4.z.optional(v4.z.array(v4.z.string())),
  useContext: v4.z.optional(v4.z.array(v4.z.string())),
  useDebugValue: v4.z.optional(v4.z.array(v4.z.string())),
  useDeferredValue: v4.z.optional(v4.z.array(v4.z.string())),
  useEffect: v4.z.optional(v4.z.array(v4.z.string())),
  useFormStatus: v4.z.optional(v4.z.array(v4.z.string())),
  useId: v4.z.optional(v4.z.array(v4.z.string())),
  useImperativeHandle: v4.z.optional(v4.z.array(v4.z.string())),
  useInsertionEffect: v4.z.optional(v4.z.array(v4.z.string())),
  useLayoutEffect: v4.z.optional(v4.z.array(v4.z.string())),
  useMemo: v4.z.optional(v4.z.array(v4.z.string())),
  useOptimistic: v4.z.optional(v4.z.array(v4.z.string())),
  useReducer: v4.z.optional(v4.z.array(v4.z.string())),
  useRef: v4.z.optional(v4.z.array(v4.z.string())),
  useState: v4.z.optional(v4.z.array(v4.z.string())),
  useSyncExternalStore: v4.z.optional(v4.z.array(v4.z.string())),
  useTransition: v4.z.optional(v4.z.array(v4.z.string()))
});
var ESLintReactSettingsSchema = v4.z.object({
  /**
   * The source where React is imported from.
   * @description This allows to specify a custom import location for React when not using the official distribution.
   * @default `"react"`
   * @example `"@pika/react"`
   */
  importSource: v4.z.optional(v4.z.string()),
  /**
   * The identifier that's used for JSX Element creation.
   * @default `"createElement"`
   * @deprecated
   */
  jsxPragma: v4.z.optional(v4.z.string()),
  /**
   * The identifier that's used for JSX fragment elements.
   * @description This should not be a member expression (i.e. use "Fragment" instead of "React.Fragment").
   * @default `"Fragment"`
   * @deprecated
   */
  jsxPragmaFrag: v4.z.optional(v4.z.string()),
  /**
   * The name of the prop that is used for polymorphic components.
   * @description This is used to determine the type of the component.
   * @example `"as"`
   */
  polymorphicPropName: v4.z.optional(v4.z.string()),
  /**
   * @default `true`
   * @internal
   */
  strict: v4.z.optional(v4.z.boolean()),
  /**
   * Check both the shape and the import to determine if an API is from React.
   * @default `true`
   * @internal
   */
  skipImportCheck: v4.z.optional(v4.z.boolean()),
  /**
   * React version to use, "detect" means auto detect React version from the project's dependencies.
   * If `importSource` is specified, an equivalent version of React should be provided here.
   * @example `"18.3.1"`
   * @default `"detect"`
   */
  version: v4.z.optional(v4.z.string()),
  /**
   * A object to define additional hooks that are equivalent to the built-in React Hooks.
   * @description ESLint React will recognize these aliases as equivalent to the built-in hooks in all its rules.
   * @example `{ useEffect: ["useIsomorphicLayoutEffect"] }`
   */
  additionalHooks: v4.z.optional(CustomHooksSchema),
  /**
   * An array of user-defined components
   * @description This is used to inform the ESLint React plugins how to treat these components during checks.
   * @example `[{ name: "Link", as: "a", attributes: [{ name: "to", as: "href" }, { name: "rel", defaultValue: "noopener noreferrer" }] }]`
   */
  additionalComponents: v4.z.optional(v4.z.array(CustomComponentSchema))
});
var ESLintSettingsSchema = v4.z.optional(
  v4.z.object({
    "react-x": v4.z.optional(v4.z.unknown())
  })
);
function isESLintSettings(settings) {
  return ESLintSettingsSchema.safeParse(settings).success;
}
function isESLintReactSettings(settings) {
  return ESLintReactSettingsSchema.safeParse(settings).success;
}
var DEFAULT_ESLINT_REACT_SETTINGS = {
  version: "detect",
  importSource: "react",
  strict: true,
  skipImportCheck: true,
  polymorphicPropName: "as",
  additionalComponents: [],
  additionalHooks: {
    useEffect: ["useIsomorphicLayoutEffect"],
    useLayoutEffect: ["useIsomorphicLayoutEffect"]
  }
};
var DEFAULT_ESLINT_SETTINGS = {
  "react-x": DEFAULT_ESLINT_REACT_SETTINGS
};
var coerceESLintSettings = (settings) => {
  return settings;
};
var decodeESLintSettings = (settings) => {
  if (isESLintSettings(settings)) {
    return settings;
  }
  return DEFAULT_ESLINT_SETTINGS;
};
var coerceSettings = (settings) => {
  return settings;
};
var decodeSettings = (settings) => {
  if (isESLintReactSettings(settings)) {
    return settings;
  }
  return DEFAULT_ESLINT_REACT_SETTINGS;
};
var normalizeSettings = ({
  additionalComponents = [],
  additionalHooks = {},
  importSource = "react",
  polymorphicPropName = "as",
  skipImportCheck = true,
  strict = true,
  version,
  ...rest
}) => {
  return {
    ...rest,
    components: additionalComponents.map((component) => {
      const { name, as = name, attributes = [], ...rest2 } = component;
      const re = kit.RegExp.toRegExp(name);
      return {
        ...rest2,
        name,
        re,
        as,
        attributes: attributes.map(({ name: name2, as: as2 = name2, ...rest3 }) => ({
          ...rest3,
          name: name2,
          as: as2
        }))
      };
    }),
    additionalHooks,
    importSource,
    polymorphicPropName,
    skipImportCheck,
    strict,
    version: tsPattern.match(version).with(tsPattern.P.union(tsPattern.P.nullish, "", "detect"), () => getReactVersion("19.1.0")).otherwise(eff.identity)
  };
};
var cache = /* @__PURE__ */ new Map();
function getSettingsFromContext(context) {
  const settings = context.settings;
  return eff.getOrElseUpdate(
    cache,
    settings["react-x"],
    () => normalizeSettings(decodeSettings(settings["react-x"]))
  );
}
var defineSettings = eff.identity;

exports.CustomComponentPropSchema = CustomComponentPropSchema;
exports.CustomComponentSchema = CustomComponentSchema;
exports.CustomHooksSchema = CustomHooksSchema;
exports.DEFAULT_ESLINT_REACT_SETTINGS = DEFAULT_ESLINT_REACT_SETTINGS;
exports.DEFAULT_ESLINT_SETTINGS = DEFAULT_ESLINT_SETTINGS;
exports.ESLintReactSettingsSchema = ESLintReactSettingsSchema;
exports.ESLintSettingsSchema = ESLintSettingsSchema;
exports.GITHUB_URL = GITHUB_URL;
exports.NPM_SCOPE = NPM_SCOPE;
exports.WEBSITE_URL = WEBSITE_URL;
exports.coerceESLintSettings = coerceESLintSettings;
exports.coerceSettings = coerceSettings;
exports.decodeESLintSettings = decodeESLintSettings;
exports.decodeSettings = decodeSettings;
exports.defineSettings = defineSettings;
exports.getDocsUrl = getDocsUrl;
exports.getId = getId;
exports.getReactVersion = getReactVersion;
exports.getSettingsFromContext = getSettingsFromContext;
exports.isESLintReactSettings = isESLintReactSettings;
exports.isESLintSettings = isESLintSettings;
exports.normalizeSettings = normalizeSettings;
